import argparse
import logging

from blackjack_gui import cli, gui
from blackjack_gui.lib import get_rules


def main():
    parser = argparse.ArgumentParser(
        description="Blackjack", epilog="Play responsibly!"
    )
    parser.add_argument("--cli", action="store_true", help="Open CLI version.")
    parser.add_argument("--ai", action="store_true", help="Computer play.")
    parser.add_argument(
        "--count",
        action="store_true",
        help="Count cards. Default is False. Can be used with --ai.",
    )
    parser.add_argument(
        "--bet", type=int, default=1, help="Bet size (1-100). Default is 1."
    )
    parser.add_argument(
        "--stack", type=int, default=1000, help="Stack size. Default is 1000."
    )
    parser.add_argument(
        "--n-games",
        type=int,
        default=10,
        help="Number of rounds to be played. Default is 10.",
    )
    parser.add_argument(
        "--loglevel",
        type=str,
        default="DEBUG",
        help="Log level. Can be DEBUG or INFO. Default is DEBUG.",
    )
    parser.add_argument(
        "--cards",
        type=lambda s: s.split(","),
        default=None,
        help="Determine first player cards. E.g. --cards=A,7 to simulate soft 18.",
    )
    parser.add_argument(
        "--dealer-cards",
        type=lambda s: s.split(","),
        default=None,
        help="Determine first dealer cards. Useful for tests.",
    )
    parser.add_argument(
        "--subset",
        type=str,
        choices=["hard", "soft", "pairs", "hard/soft", "soft/pairs"],
        help="Subset of hands to be played. Default is all.",
        default=None,
    )
    parser.add_argument(
        "--rules",
        type=str,
        choices=["US", "Helsinki"],
        help="Rules to be used. US = H17, peek, no surrender. Helsinki = S17, no peek, surrender.",
        default="Helsinki",
    )

    args = parser.parse_args()

    args.rules = get_rules(args.rules)

    if args.cli:
        logging.basicConfig(level=args.loglevel, format="%(message)s")
        cli.play(args)
    else:
        logging.basicConfig(level="WARNING")
        gui.main(args)


if __name__ == "__main__":
    main()



from argparse import Namespace
import os
import tkinter
from dataclasses import dataclass
from typing import Any


from PIL import Image, ImageTk

from .lib import (
    Card,
    Dealer,
    Hand,
    Player,
    Shoe,
    get_correct_play,
    get_starting_hand,
)

from .table_components import TableComponents, get_image, CheckButton

TIME_DELAY = 800
N_CARDS_MAX = 11
IMG_PATH = f"{os.path.dirname(__file__)}/images/"


@dataclass
class Gui:
    root: tkinter.Tk
    menu: dict
    label_text: tkinter.StringVar
    slot_player: dict
    slot_dealer: dict
    info_text: dict
    info: dict
    chips: dict
    finger: dict
    shoe_progress: tkinter.Label
    fix_mistakes: tkinter.IntVar
    slider: tkinter.Scale
    insurance_chip: tkinter.Label
    dealer_info: tkinter.Label
    accuracy_text: tkinter.StringVar
    show_count: tkinter.IntVar
    count_text: tkinter.StringVar
    use_csm: tkinter.IntVar


class Game:
    def __init__(self, player: Player, dealer: Dealer, gui: Gui, args: Any):
        self.player = player
        self.dealer = dealer
        self.gui = gui
        self.args = args
        self.bet = args.bet
        self.shoe = Shoe(6)
        self.active_slot = None
        self.initial_bet = args.bet
        self._n_correct_play = 0
        self._n_mistakes = 0
        self._n_rounds = 0

    def start_new_round(self):
        """Starts new round."""
        self._hide_buttons()
        self._n_rounds += 1
        self._update_accuracy()
        self.bet = self.gui.slider.get()
        self.gui.slider.configure(state=tkinter.DISABLED)
        self._hide_all_chips()
        self._hide_insurance_chip()
        self._hide_fingers()
        self._clean_player_slots()
        self._dealer_info()
        self.player.hands = []
        if (
            (self.gui.use_csm.get() == 1)
            or self.shoe.n_cards < 52
            or self.args.cards is not None
            or self.args.subset is not None
        ):
            self.shoe = Shoe(6)
            self.player.init_count()

        hand = self.player.start_new_hand(self.bet)
        self.dealer.init_hand()
        if self.args.dealer_cards is not None:
            self.shoe.arrange(self.args.dealer_cards)
        self.dealer.deal(self.shoe, self.gui.shoe_progress)
        self.dealer.deal(self.shoe, self.gui.shoe_progress)
        self.dealer.cards[1].visible = False
        self._display_dealer_cards()
        self._handle_counts(self.dealer.cards, self.shoe)
        if self.args.cards is not None:
            self.shoe.arrange(self.args.cards, randomize=True)
        elif self.args.subset is not None:
            cards = get_starting_hand(self.args.subset)
            self.shoe.arrange(cards)
        hand.deal(self.shoe, self.gui.shoe_progress)
        hand.deal(self.shoe, self.gui.shoe_progress)
        self._handle_counts(hand, self.shoe)
        self._show()
        self.active_slot = hand.slot
        self._display_stack()
        self._display_chip(hand, 0)
        self._display_player_cards(hand)
        if self.dealer.has_ace:
            self._enable_correct_buttons(hand)
            if hand.is_blackjack is True:
                self._hide_buttons(("hit", "double"))
                self._show_buttons(("even-money",))
            else:
                self._show_buttons(("insurance",))
        else:
            if hand.is_blackjack:
                self.gui.root.after(TIME_DELAY, self._end_round)
                return
            self._enable_correct_buttons(hand)
            if self.args.rules.surrender:
                self._show_buttons(("surrender",))

    def surrender(self):
        """Surrender button."""
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_play(self.player.hands[0], "surrender") is False
        ):
            return
        self._hide_buttons()
        self.player.stack += self.bet / 2
        self._display_stack()
        hand = self._get_hand_in_active_slot()
        self.gui.root.after(TIME_DELAY, self._reveal_dealer_hidden_card, True)
        self._hide(hand)
        self._hide_chips(hand)
        self._display_info(hand, "SURRENDER")

    def even_money(self):
        """Even Money button"""
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_insurance(hand) is False
        ):
            return
        self.dealer.even_money = True
        self._hide(hand)
        self._payout()

    def double(self):
        """Double button."""
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_play(hand, "double") is False
        ):
            return
        if self._check_dealer_peek():
            return
        self._hide_buttons(("surrender",))
        self.player.stack -= self.bet
        self._display_stack()
        hand.bet += self.bet
        hand.deal(self.shoe, self.gui.shoe_progress)
        self._display_chip(hand, 1)
        hand.is_finished = True
        self._display_player_cards(hand, rotate_last=True)
        if hand.is_triple_seven:
            self._hide_buttons()
            self.gui.root.after(TIME_DELAY, self._end_round)
            return
        self._handle_counts(hand, self.shoe)
        if hand.sum > 21:
            self._hide(hand)
            self._hide_chips(hand)
        self._clean_info()
        self._resolve_next_hand()

    def reset(self):
        """Reset button."""
        self._clean_info()
        self.player.buy_in(self.player.initial_stack)
        self.shoe = Shoe(6)
        self._clean_dealer_slots()
        self.gui.slider.set(self.initial_bet)
        self.player.init_count()
        self._reset_accuracy()
        self.start_new_round()

    def deal(self):
        """Deal button."""
        self._clean_info()
        self._clean_dealer_slots()
        self.start_new_round()

    def hit(self):
        """Hit button."""
        self._hide_buttons(("insurance",))
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_play(hand, "hit") is False
        ):
            return
        if self._check_dealer_peek():
            return
        self._hide_buttons(("surrender", "double"))
        hand.deal(self.shoe, self.gui.shoe_progress)
        self._display_player_cards(hand)
        if hand.is_triple_seven:
            self._hide_buttons()
            self.gui.root.after(TIME_DELAY, self._end_round)
            return
        self._handle_counts(hand, self.shoe)
        if hand.is_over is True:
            self._hide(hand)
            self._hide_chips(hand)
            self._display_info(hand, "BUST")
        if hand.is_finished is False:
            self._enable_correct_buttons(hand)
        else:
            self._resolve_next_hand()

    def stay(self):
        """Stay button."""
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_play(hand, "stay") is False
        ):
            return
        hand.is_finished = True
        self._resolve_next_hand()

    def insurance(self):
        """Insurance button."""
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_insurance(hand) is False
        ):
            return
        self.dealer.insurance_bet = hand.bet / 2
        self._display_insurance_chip()
        self.player.stack -= self.dealer.insurance_bet
        self._display_stack()
        self._hide_buttons(("insurance",))
        self._check_dealer_peek()

    def split(self):
        """Split button."""
        hand = self._get_hand_in_active_slot()
        if (
            self.gui.fix_mistakes.get() == 1
            and self._check_play(hand, "split") is False
        ):
            return
        if self._check_dealer_peek():
            return
        self._hide_buttons(("surrender", "insurance"))
        new_hand = self.player.start_new_hand(self.bet)
        split_card = hand.cards.pop()
        new_hand.deal(split_card, self.gui.shoe_progress)
        self._display_chip(new_hand, 0)
        self._display_stack()
        for handy in (hand, new_hand):
            handy.is_split_hand = True
            handy.deal(self.shoe, self.gui.shoe_progress)
            if handy.cards[0].label == "A":
                # Split Aces receive only one card more
                handy.is_hittable = False
                handy.is_finished = True

        self.player.sort_hands()
        if len(self.player.hands) < 4:
            self.player.hands.sort(
                key=lambda x: not x.cards[0].value == x.cards[1].value
            )
        for hand in self.player.hands:
            rotate = hand.cards[0].label == "A" and hand.cards[1].label != "A"
            self._display_player_cards(hand, rotate_last=rotate)
            self._handle_counts(hand, self.shoe)
        self._resolve_next_hand()

    def _resolve_next_hand(self):
        hand = self._get_first_unfinished_hand()
        if hand is not None:
            self.active_slot = hand.slot
            self._enable_correct_buttons(hand)
            self._display_finger(hand)
        else:
            self._clean_info()
            self._hide_buttons()
            if not self._is_all_over() or self.dealer.insurance_bet > 0:
                self.gui.root.after(TIME_DELAY, self._reveal_dealer_hidden_card)
            else:
                self._handle_counts(self.dealer.cards, self.shoe)
                self._payout()

    def _reveal_dealer_hidden_card(self, surrender: bool = False):
        self._display_dealer_cards(hide_second=False)
        self.dealer.cards[1].visible = True
        self._handle_counts(self.dealer.cards, self.shoe)
        if surrender:
            self._show_buttons(("deal",))
            self.gui.slider.configure(state=tkinter.NORMAL)
        else:
            self._check_dealer_blackjack()

    def _check_dealer_blackjack(self):
        if self.dealer.is_blackjack and self.dealer.insurance_bet > 0:
            self._display_insurance_chip(triple=True)
        else:
            self._hide_insurance_chip()

        if not self.dealer.is_finished:
            self.gui.root.after(TIME_DELAY, self._dealer_draw_one_card)
        else:
            self._handle_counts(self.dealer.cards, self.shoe)
            self._payout()

    def _dealer_draw_one_card(self):
        self.dealer.deal(self.shoe, self.gui.shoe_progress)
        self._display_dealer_cards()
        self._handle_counts(self.dealer.cards, self.shoe)
        if not self.dealer.is_finished:
            self.gui.root.after(TIME_DELAY, self._dealer_draw_one_card)
        else:
            self._handle_counts(self.dealer.cards, self.shoe)
            self._payout()

    def _payout(self):
        """Handles payout of all hands."""
        self._hide_fingers()
        for hand in self.player.hands:
            if (
                self.dealer.insurance_bet > 0
                and self.dealer.is_blackjack
                and hand.slot == 2
            ):
                self.player.stack += self.dealer.insurance_bet * 3
                self._display_insurance_chip(triple=True)
                if hand.is_triple_seven:
                    self.player.stack += hand.bet * 3
                    result = "TRIPLE SEVEN + INSURANCE"
                    if hand.bet == 2 * self.bet:
                        self.bet = hand.bet
                    self._display_chips(hand, triple=True)
                else:
                    self._hide_all_chips()
                    result = "INSURANCE"
            elif self.dealer.even_money is True:
                self.player.stack += hand.bet * 2
                result = "EVEN MONEY"
                self._display_chips(hand)
            elif hand.is_triple_seven is True:
                self.player.stack += hand.bet * 3
                result = "TRIPLE SEVEN"
                if hand.bet == 2 * self.bet:
                    self.bet = hand.bet
                self._display_chips(hand, triple=True)
            elif (
                hand.is_blackjack is True and self.dealer.is_blackjack is False
            ):
                self.player.stack += hand.bet * 2.5
                result = "BLACKJACK"
                self._display_chips(hand, bj=True)
            elif hand.is_blackjack is True and self.dealer.is_blackjack is True:
                self.player.stack += hand.bet
                result = "PUSH"
            elif (
                self.dealer.is_blackjack is True
                and hand.is_blackjack is False
                and hand.is_over is False
            ):
                self._dealer_info("BLACKJACK")
                result = "LOSE"
                self._resolve_lost_hand(hand)
            elif hand.is_over is False and self.dealer.is_over is True:
                self._dealer_info("BUST")
                self.player.stack += hand.bet * 2
                result = ""
                self._display_chips(hand)
            elif hand.is_over is True:
                result = "BUST"
                self._resolve_lost_hand(hand)
            elif hand.sum < self.dealer.sum:
                result = f"LOSE ({hand.sum} vs {self.dealer.sum})"
                self._resolve_lost_hand(hand)
            elif hand.surrender is True:
                self.player.stack += hand.bet / 2
                result = ""
            elif hand.sum > self.dealer.sum:
                self.player.stack += hand.bet * 2
                result = f"WIN ({hand.sum} vs {self.dealer.sum})"
                self._display_chips(hand)
            elif hand.sum == self.dealer.sum:
                self.player.stack += hand.bet
                result = "PUSH"
            else:
                raise ValueError
            self._display_info(hand, result)
            self._handle_counts(hand, self.shoe)
        # show both dealer cards even if player busts
        if (
            len(self.dealer.cards) == 2
            and self.dealer.cards[1].visible is False
        ):
            self.dealer.is_finished = True
            self.gui.root.after(
                TIME_DELAY, self._reveal_dealer_hidden_card, True
            )
        else:
            self._show_buttons(("deal",))
            self._handle_counts(self.dealer.cards, self.shoe)
            self.gui.slider.configure(state=tkinter.NORMAL)

    def _handle_counts(self, hand: Hand | list[Card], shoe: Shoe):
        self.player.update_counts(hand, shoe)
        true_count = int(self.player.true_count)
        self.gui.count_text.set(
            f"Running count: {self.player.running_count}\nTrue count: {true_count}"
        )

    def _resolve_lost_hand(self, hand: Hand):
        self._hide_chips(hand)
        self._hide(hand)
        hand.bet = 0

    def _check_dealer_peek(self) -> bool:
        if self.args.rules.peek and self.dealer.is_blackjack:
            self._hide_buttons()
            self.gui.root.after(TIME_DELAY, self._reveal_dealer_hidden_card)
            return True
        return False

    def _end_round(self):
        self._display_dealer_cards(hide_second=False)
        self.dealer.cards[1].visible = True
        self._payout()

    def _enable_correct_buttons(self, hand: Hand):
        n_hands = len(self.player.hands)
        if len(hand.cards) == 2 and hand.is_hittable is True:
            self._show_buttons(("double",))
        else:
            self._hide_buttons(("double",))
        if (
            hand.cards[0].value == hand.cards[1].value
            and len(hand.cards) == 2
            and n_hands < 4
        ):
            self._show_buttons(("split",))
        else:
            self._hide_buttons(("split",))
        if hand.is_hittable is True:
            self._show_buttons(("hit", "stay"))
        else:
            self._hide_buttons(("hit", "stay"))

    def _check_play(self, hand: Hand, play: str) -> bool:
        """Verifies player decision. Ignores deviations."""
        correct_play = get_correct_play(
            hand, self.dealer.cards[0], len(self.player.hands), self.args.rules
        )
        if correct_play != play:
            self._display_info(hand, "Try again!")
            self.gui.root.after(1000, self._clean_info)
            self._n_mistakes += 1
            return False
        self._n_correct_play += 1
        if self.gui.fix_mistakes.get() == 1:
            self._update_accuracy()
        return True

    def _update_accuracy(self):
        n_decisions = self._n_correct_play + self._n_mistakes
        if n_decisions != 0:
            txt = f"Accuracy: {round(self._n_correct_play / (self._n_correct_play + self._n_mistakes) * 100, 2)}%"
        else:
            txt = "Accuracy: 0%"
        txt += f"\nRounds: {self._n_rounds}"
        self.gui.accuracy_text.set(txt)

    def _check_insurance(self, hand: Hand) -> bool:
        if self.player.true_count < 3:
            self._display_info(hand, "Try again!")
            self.gui.root.after(1000, self._clean_info)
            return False
        return True

    def _display_stack(self):
        unit = "$" if self.args.rules.region == "US" else "€"
        self.gui.label_text.set(f"Stack: {self.player.stack} {unit}")

    def _display_chips(self, hand, bj: bool = False, triple: bool = False):
        if bj is True:
            self._display_chip(hand, 1)
            self._display_chip(hand, 4, color="blue")
        elif triple is True:
            self._display_chip(hand, 0)
            self._display_chip(hand, 1)
            self._display_chip(hand, 2)
        elif hand.bet == self.bet:
            self._display_chip(hand, 1)
        elif hand.bet == (2 * self.bet):
            self._display_chip(hand, 2)
            self._display_chip(hand, 3)

    def _is_all_over(self) -> bool:
        for hand in self.player.hands:
            if hand.is_over is False and hand.surrender is False:
                return False
        return True

    def _get_first_unfinished_hand(self) -> Hand | None:
        for hand in self.player.hands:
            if hand.is_finished is False:
                return hand
        return None

    def _get_hand_in_active_slot(self) -> Hand:
        for hand in self.player.hands:
            if hand.slot == self.active_slot:
                return hand
        raise RuntimeError

    def _show(self):
        for slot in range(4):
            for n in range(N_CARDS_MAX):
                self.gui.slot_player[f"{str(slot)}{str(n)}"].configure(
                    state=tkinter.NORMAL
                )

    def _hide(self, hand: Hand):
        for n in range(N_CARDS_MAX):
            self.gui.slot_player[f"{str(hand.slot)}{str(n)}"].configure(
                state=tkinter.DISABLED
            )

    def _hide_buttons(self, buttons: tuple | None = None):
        if buttons is None:
            for key, button in self.gui.menu.items():
                if key != "reset":
                    button.configure(state=tkinter.DISABLED)
        else:
            for button in buttons:
                if button in self.gui.menu.keys():
                    self.gui.menu[button].configure(state=tkinter.DISABLED)

    def _show_buttons(self, buttons: tuple | None = None):
        if buttons is None:
            for key, button in self.gui.menu.items():
                if key not in ("insurance", "even-money"):
                    button.configure(state=tkinter.NORMAL)
        else:
            for button in buttons:
                if button in self.gui.menu.keys():
                    self.gui.menu[button].configure(state=tkinter.NORMAL)

    def _clean_player_slots(self):
        for slot in range(4):
            for n in range(N_CARDS_MAX):
                self.gui.slot_player[f"{str(slot)}{str(n)}"].configure(
                    image="", width=0, height=0
                )

    def _clean_dealer_slots(self):
        for pos in self.gui.slot_dealer.values():
            pos.configure(image="", width=0)

    def _clean_info(self):
        for slot in range(4):
            self.gui.info_text[str(slot)].set("")

    def _display_dealer_cards(self, hide_second: bool = True):
        for ind, card in enumerate(self.dealer.cards):
            if ind == 1 and hide_second is True and len(self.dealer.cards) == 2:
                img, width, _ = get_image()
            else:
                img, width, _ = get_image(card)
            self.gui.slot_dealer[str(ind)].configure(image=img, width=width)
            self.gui.slot_dealer[str(ind)].image = img

    def _display_player_cards(self, hand: Hand, rotate_last: bool = False):
        for ind, card in enumerate(hand.cards):
            rotate = ind == len(hand.cards) - 1 and rotate_last is True
            img, width, height = get_image(card, rotate=rotate)
            self.gui.slot_player[f"{str(hand.slot)}{str(ind)}"].configure(
                image=img, width=width, height=height
            )
            self.gui.slot_player[f"{str(hand.slot)}{str(ind)}"].image = img

    def _display_insurance_chip(self, triple: bool = False):
        bet = (
            self.dealer.insurance_bet
            if triple is False
            else self.dealer.insurance_bet * 3
        )
        color = "red"
        if bet == 0.5:
            color = "blue"
            text = "0.5"
        elif bet % 1 == 0:
            text = str(round(bet))
        else:
            text = str(bet)
        img = _get_chip_image(color)
        self.gui.insurance_chip.configure(
            image=img,
            compound="center",
            fg="white",
            text=text,
            font="helvetica 10 bold",
        )
        self.gui.insurance_chip.image = img  # type: ignore

    def _hide_insurance_chip(self):
        self.gui.insurance_chip.configure(image="", text="")

    def _display_chip(self, hand: Hand, pos: int, color: str = "red"):
        img = _get_chip_image(color)
        if color == "red":
            text = self.bet
        else:
            text = ".5" if self.bet == 1 else self.bet / 2
        self.gui.chips[f"{str(hand.slot)}{str(pos)}"].configure(
            image=img,
            compound="center",
            fg="white",
            text=text,
            font="helvetica 10 bold",
        )
        self.gui.chips[f"{str(hand.slot)}{str(pos)}"].image = img

    def _display_finger(self, hand: Hand):
        self._hide_fingers()
        img = _get_finger_image()
        self.gui.finger[f"{str(hand.slot)}"].configure(image=img)
        self.gui.finger[f"{str(hand.slot)}"].image = img

    def _dealer_info(self, text: str = ""):
        self.gui.dealer_info.configure(text=text)

    def _hide_chips(self, hand: Hand):
        for pos in range(4):
            self.gui.chips[f"{str(hand.slot)}{str(pos)}"].configure(
                image="", text=""
            )

    def _hide_all_chips(self):
        for chip in self.gui.chips.values():
            chip.configure(image="", text="")

    def _hide_fingers(self):
        for finger in self.gui.finger.values():
            finger.configure(image="")

    def _display_info(self, hand: Hand, info: str):
        self.gui.info_text[str(hand.slot)].set(info)

    def _reset_accuracy(self):
        self._n_correct_play = 0
        self._n_mistakes = 0
        self._n_rounds = 0
        self.gui.accuracy_text.set("")


def _get_chip_image(color: str = "red") -> ImageTk.PhotoImage:
    size = 50
    filename = f"{IMG_PATH}/{color}-chip.png"
    image = Image.open(filename).resize(
        (size, size - 15), Image.Resampling.LANCZOS
    )
    return ImageTk.PhotoImage(image)


def _get_finger_image() -> ImageTk.PhotoImage:
    filename = f"{IMG_PATH}/finger2.png"
    image = Image.open(filename).resize((40, 60), Image.Resampling.LANCZOS)
    return ImageTk.PhotoImage(image)


def main(args: Namespace):
    root = tkinter.Tk()
    root.geometry("1200x700")
    rules = (
        "H17, Peek, No Surrender, DAS"
        if args.rules.region == "US"
        else "S17, No Peek, Surrender, DAS, 7-7-7 pays 3:1"
    )
    root.title(f"Blackjack {args.rules.region} ({rules})")
    background = "#4e9572"
    root.configure(background=background)

    components = TableComponents(root, background)
    components.setup_canvas()
    shoe_progress = components.get_shoe_status()
    label_text = components.get_label()
    dealer_info = components.get_dealer_info()
    info, info_text = components.get_info()
    finger = components.get_finger()
    slot_player = components.get_player_slots(N_CARDS_MAX)
    chips = components.get_chips()
    slot_dealer = components.get_dealer_slot()
    insurance_chip = components.get_insurance_chip()
    components.set_side_panel()

    check_button = CheckButton(root, args, background)
    accuracy_text, fix_mistakes = check_button.fetch_accuracy()
    count_text, fix_count = check_button.fetch_count()
    use_csm = check_button.fetch_csm()

    # Buttons
    menu = {
        name.split()[0].lower(): tkinter.Button(
            master=root, text=name.replace("-", " "), width=12, font="15"
        )
        for name in (
            "Even-money",
            "Insurance",
            "Surrender",
            "Double up",
            "Hit",
            "Stay",
            "Split",
            "Deal",
            "Reset",
        )
    }
    for name, button in menu.items():
        if name == "hit":
            button.configure(command=lambda: game.hit())
        elif name == "split":
            button.configure(command=lambda: game.split())
        elif name == "surrender":
            button.configure(command=lambda: game.surrender())
        elif name == "stay":
            button.configure(command=lambda: game.stay())
        elif name == "double":
            button.configure(command=lambda: game.double())
        elif name == "deal":
            button.configure(command=lambda: game.deal())
        elif name == "reset":
            button.configure(command=lambda: game.reset())
        elif name == "insurance":
            button.configure(command=lambda: game.insurance())
        elif name == "even-money":
            button.configure(command=lambda: game.even_money())
        else:
            raise ValueError
    x_sidepanel = 1025
    for ind, button in enumerate(menu.values()):
        button.place(x=x_sidepanel, y=ind * 33 + 230)
    menu["deal"].place(x=x_sidepanel, y=500)
    menu["reset"].place(x=x_sidepanel, y=20)

    slider = components.get_slider(x_sidepanel, args.bet)

    gui = Gui(
        root,
        menu,
        label_text,
        slot_player,
        slot_dealer,
        info_text,
        info,
        chips,
        finger,
        shoe_progress,
        fix_mistakes,
        slider,
        insurance_chip,
        dealer_info,
        accuracy_text,
        fix_count,
        count_text,
        use_csm,
    )

    dealer = Dealer(args.rules.game_type)
    player = Player(
        rules=args.rules,
        stack=args.stack,
    )
    game = Game(player, dealer, gui, args)
    game.reset()
    tkinter.mainloop()



from dataclasses import dataclass
import random
import tkinter
from typing import List, Literal


@dataclass
class Rules:
    game_type: Literal["s17", "h17"]
    surrender: bool
    peek: bool
    double_after_split: bool = True
    triple_seven: bool = False
    region: Literal["US", "Europe", "Helsinki"] = "US"


class Card:
    def __init__(self, label: str, suit: str, visible: bool = True):
        self.label = label
        self.suit = suit
        self.value = self._get_value()
        self.visible = visible
        self.counted = False

    def _get_value(self) -> int | tuple:
        if self.label in ("2", "3", "4", "5", "6", "7", "8", "9", "10"):
            return int(self.label)
        if self.label in ("J", "Q", "K"):
            return 10
        if self.label == "A":
            return 1, 11
        raise ValueError("Bad label")

    def __repr__(self) -> str:
        if self.suit == "spades":
            suit = "\u2660"
        elif self.suit == "clubs":
            suit = "\u2663"
        elif self.suit == "diamonds":
            suit = "\u2666"
        elif self.suit == "hearts":
            suit = "\u2665"
        else:
            raise ValueError("Bad suit")
        return f"{self.label}{suit}"


class Deck:
    def __init__(self):
        self.cards = []
        self._build()

    def _build(self):
        for suit in ["spades", "clubs", "diamonds", "hearts"]:
            for v in (
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
                "10",
                "J",
                "Q",
                "K",
                "A",
            ):
                self.cards.append(Card(v, suit))


class Shoe:
    def __init__(self, n_decs: int):
        self.cards: List[Card] = []
        self.n_cards = 0
        self.n_decs = n_decs
        self._n_cards_total = self.n_decs * 52
        self._build()

    def _build(self):
        for _ in range(self.n_decs):
            deck = Deck()
            for card in deck.cards:
                self.cards.append(card)
        random.shuffle(self.cards)
        self.n_cards = len(self.cards)

    def draw(self, progress: tkinter.Label | None = None) -> Card:
        """Draws a card from shoe."""
        if self.n_cards > 0:
            card = self.cards.pop(0)
            self.n_cards -= 1
            fraction = (
                self._n_cards_total - self.n_cards
            ) / self._n_cards_total
            if progress is not None:
                progress.place(
                    x=30, y=150, anchor="se", relheight=fraction, relwidth=1.0
                )
            return card
        raise ValueError("Empty shoe!")

    def arrange(self, cards: list, randomize: bool = False):
        """Arranges shoe so that next cards are the requested ones."""
        if ";" in str(cards):
            # Choose one hand randomly from input like --cards="A,7;7,7;10,10"
            options = [
                [cards[i].split(";")[-1], cards[i + 1].split(";")[0]]
                for i in range(len(cards) - 1)
            ]
            cards = random.choice(options)

        labels = [card.label for card in self.cards]
        if randomize and len(cards) > 1:
            # randomize the first two cards
            cards = random.sample(cards[0:2], 2) + cards[2:]
        for ind, card in enumerate(cards):
            indices = [i for i, x in enumerate(labels[ind:]) if x == str(card)]
            shoe_ind = random.choice(indices) + ind
            self.cards[shoe_ind], self.cards[ind] = (
                self.cards[ind],
                self.cards[shoe_ind],
            )
            labels[shoe_ind], labels[ind] = labels[ind], labels[shoe_ind]


class Hand:
    def __init__(self, rules: Rules):
        self.rules = rules
        self.cards: list[Card] = []
        self.sum = 0.0
        self.bet = 0.0
        self.is_hard = True
        self.is_hittable = True  # if True, can receive more cards
        self.is_blackjack = False
        self.is_over = False
        self.surrender = False
        self.is_asked_to_split = False
        self.is_split_hand = False
        self.slot = None
        self.is_finished = False  # if True, no more playing for this hand
        self.played = False
        self.is_triple_seven = False

    def deal(
        self,
        source: Shoe | Card,
        progress: tkinter.Label | None = None,
    ):
        if isinstance(source, Shoe):
            self.cards.append(source.draw(progress))
        else:
            self.cards.append(source)
        self.sum, self.is_hard = evaluate_hand(self.cards)

        if (
            len(self.cards) == 3
            and all(card.label == "7" for card in self.cards)
            and self.is_split_hand is False
            and self.rules.triple_seven
        ):
            self.is_triple_seven = True
            self.is_finished = True
            self.is_hittable = False
        if self.sum >= 22:
            self.is_finished = True
            self.is_hittable = False
        if self.sum > 21:
            self.is_over = True
        if (
            self.sum == 21
            and len(self.cards) == 2
            and self.is_split_hand is False
        ):
            self.is_blackjack = True

    def __repr__(self) -> str:
        return format_hand(self.cards)


class Dealer:
    def __init__(self, game_type: Literal["h17", "s17"]):
        self.cards: list[Card] = []
        self.sum = 0.0
        self.is_blackjack = False
        self.is_finished = False
        self.is_over = False
        self.insurance_bet = 0.0
        self.even_money = False
        self.game_type = game_type
        self.has_ace = False

    def init_hand(self):
        self.cards = []
        self.sum = 0
        self.is_blackjack = False
        self.is_finished = False
        self.is_over = False
        self.insurance_bet = 0
        self.even_money = False

    def deal(self, shoe: Shoe, progress: tkinter.Label | None = None):
        card = shoe.draw(progress)
        self.cards.append(card)
        self.sum, _ = evaluate_hand(self.cards)
        labels = [c.label for c in self.cards]
        self.has_ace = True if labels[0] == "A" else False
        if self.sum == 17 and self.game_type == "h17" and "A" in labels:
            pass
        elif self.sum > 16:
            self.is_finished = True
        if self.sum == 21 and len(self.cards) == 2:
            self.is_blackjack = True
        if self.sum > 21:
            self.is_over = True

    def __repr__(self) -> str:
        return format_hand(self.cards)


class Player:
    def __init__(self, rules: Rules, stack: float = 1000):
        self.rules = rules
        self.stack = stack
        self.hands: List[Hand] = []
        self.initial_stack = stack
        self.invested = 0.0
        self.running_count = 0
        self.true_count = 0.0

    def buy_in(self, bet: float):
        self.stack = bet

    def start_new_hand(self, bet: float) -> Hand:
        hand = Hand(self.rules)
        hand.bet = bet
        self.stack -= bet
        self.invested += bet
        hand.slot = self._get_next_free_slot()
        self.hands.append(hand)
        return hand

    def sort_hands(self):
        self.hands.sort(key=lambda x: x.slot)  # type: ignore

    def _get_next_free_slot(self):
        n_hands = len(self.hands)
        if n_hands == 0:
            return 2
        if n_hands == 1:
            return 1
        if n_hands == 2:
            return 3
        if n_hands == 3:
            return 0
        raise RuntimeError("Too many hands")

    def init_count(self):
        self.running_count = 0
        self.true_count = 0.0

    def update_true_count(self, shoe: Shoe):
        n_decs_left = shoe.n_cards / 52
        self.true_count = self.running_count / n_decs_left

    def update_running_count(self, card: Card):
        if card.visible is False or card.counted:
            return
        if card.label == "A" or card.value == 10:
            self.running_count -= 1
        elif isinstance(card.value, int) and card.value <= 6:
            self.running_count += 1
        card.counted = True

    def update_counts(self, hand: Hand | list[Card], shoe: Shoe):
        cards = hand.cards if isinstance(hand, Hand) else hand
        for card in cards:
            self.update_running_count(card)
        self.update_true_count(shoe)


def evaluate_hand(cards: list) -> tuple:
    the_sum = 0
    ace_used = False
    is_hard = True
    for card in cards:
        if card.label == "A":
            is_hard = False
            if ace_used is False:
                the_sum += 11
                ace_used = True
            else:
                the_sum += 1
        else:
            if isinstance(card.value, int):
                the_sum += card.value
    if the_sum > 21:
        the_sum = 0
        is_hard = True
        for card in cards:
            if card.label == "A":
                the_sum += 1
            else:
                if isinstance(card.value, int):
                    the_sum += card.value
    return the_sum, is_hard


def get_rules(region: Literal["US", "Europe", "Helsinki"]):
    if region == "US":
        rules = Rules(
            game_type="h17",
            surrender=False,
            peek=True,
        )
    if region == "Europe":
        rules = Rules(
            game_type="s17",
            surrender=False,
            peek=False,
        )
    if region == "Helsinki":
        rules = Rules(
            game_type="s17",
            surrender=True,
            peek=False,
            triple_seven=True,
        )
    rules.region = region
    return rules


def get_correct_play(
    hand: Hand, dealer_card: Card, n_hands: int, rules: Rules
) -> str:
    cards = hand.cards
    n_cards = len(cards)
    split = "split"
    hit = "hit"
    stay = "stay"
    surrender = "surrender"
    double = "double"
    dealer_ace = dealer_card.label == "A"

    # Hard hands
    if hand.is_hard is True and not (
        n_cards == 2 and cards[0].value == cards[1].value
    ):
        if hand.sum <= 8:
            return hit
        if hand.sum == 9:
            if (
                dealer_card.value in range(3, 7)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum == 10:
            if (
                dealer_card.value in range(2, 10)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum == 11:
            if (
                rules.region == "US"
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            if (
                dealer_card.value in range(2, 10)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum == 12:
            return stay if dealer_card.value in (4, 5, 6) else hit
        if hand.sum == 13:
            return stay if dealer_card.value in range(2, 7) else hit
        if hand.sum == 14:
            if (
                _should_surrender(hand, dealer_card, (10,)) is True
                and rules.region == "Helsinki"
            ):
                return surrender
            return stay if dealer_card.value in range(2, 7) else hit
        if hand.sum == 15:
            if (
                _should_surrender(hand, dealer_card, (10,)) is True
                and rules.region == "Helsinki"
            ):
                return surrender
            if (
                dealer_ace is False
                and isinstance(dealer_card.value, int)
                and dealer_card.value <= 6
            ):
                return stay
            return hit
        if hand.sum == 16:
            if (
                _should_surrender(hand, dealer_card, (9, 10)) is True
                and rules.surrender
            ):
                return surrender
            if n_cards >= 3 and dealer_card.value == 10:
                return stay
            if (
                dealer_ace is False
                and isinstance(dealer_card.value, int)
                and dealer_card.value <= 6
            ):
                return stay
            return hit
        if hand.sum >= 17:
            return stay

    # Pairs
    if n_cards == 2 and cards[0].value == cards[1].value:
        if cards[0].label == "A":
            if n_hands == 4:
                return hit
            if rules.region == "US":
                return split
            return hit if dealer_ace is True else split
        if cards[0].value == 10:
            return stay
        if cards[0].value == 9:
            if (
                dealer_card.value in (7, 10)
                or dealer_card.label == "A"
                or n_hands == 4
            ):
                return stay
            return split
        if cards[0].value == 8:
            if rules.region == "US":
                return split
            if _should_surrender(hand, dealer_card, (10,)) is True:
                return surrender
            if n_hands == 4 or dealer_ace is True:
                return hit
            return split
        if cards[0].value == 7:
            if (
                isinstance(dealer_card.value, int)
                and dealer_card.value <= 7
                and n_hands < 4
            ):
                return split
            return hit
        if cards[0].value == 6:
            if (
                isinstance(dealer_card.value, int)
                and dealer_card.value <= 6
                and n_hands < 4
            ):
                return split
            return hit
        if cards[0].value == 5:
            if isinstance(dealer_card.value, int) and dealer_card.value <= 9:
                if len(cards) == 2:
                    return double
            return hit
        if cards[0].value == 4:
            if dealer_card.value in (5, 6) and n_hands < 4:
                return split
            return hit
        if cards[0].value == 3:
            if (
                isinstance(dealer_card.value, int)
                and dealer_card.value <= 7
                and n_hands < 4
            ):
                return split
            return hit
        if cards[0].value == 2:
            if (
                isinstance(dealer_card.value, int)
                and dealer_card.value <= 7
                and n_hands < 4
            ):
                return split
            return hit

    # Soft hands
    if hand.is_hard is False:
        labels = [card.label for card in cards]
        assert "A" in labels
        if hand.sum > 19:
            return stay
        if hand.sum == 19:
            if (
                rules.region == "US"
                and dealer_card.value == 6
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return stay
        if hand.sum == 18:
            if rules.region == "US":
                if dealer_card.value in range(2, 7):
                    if n_cards == 2 and hand.is_hittable is True:
                        return double
                    return stay
                if dealer_card.value in (7, 8):
                    return stay
                return hit
            if dealer_card.value in range(3, 7):
                if n_cards == 2 and hand.is_hittable is True:
                    return double
                return stay
            if dealer_card.value in (2, 7, 8):
                return stay
            return hit
        if hand.sum == 17:
            if (
                dealer_card.value in range(3, 7)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum in (15, 16):
            if (
                dealer_card.value in (4, 5, 6)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum in (13, 14):
            if (
                dealer_card.value in (5, 6)
                and n_cards == 2
                and hand.is_hittable is True
            ):
                return double
            return hit
        if hand.sum < 13:
            return hit

    raise ValueError("Don't know what to do")


def _should_surrender(hand: Hand, dealer_card: Card, values: tuple) -> bool:
    if (
        hand.is_split_hand is True
        or dealer_card.label[0] == "A"
        or len(hand.cards) != 2
    ):
        return False
    if dealer_card.value in values:
        return True
    return False


def format_hand(cards: list) -> str:
    return str(cards)[1:-1].replace(",", " ") + " "


def get_starting_hand(subset: str) -> list[str]:
    hard_hands = [
        "2,3",
        "2,4",
        "2,5",
        "2,6",
        "2,7",
        "2,8",
        "2,9",
        "2,10",
        "3,4",
        "3,5",
        "3,6",
        "3,7",
        "3,8",
        "3,9",
        "3,10",
        "4,5",
        "4,6",
        "4,7",
        "4,8",
        "4,9",
        "4,10",
        "5,6",
        "5,7",
        "5,8",
        "5,9",
        "5,10",
        "6,7",
        "6,8",
        "6,9",
        "6,10",
        "7,8",
        "7,9",
        "7,10",
        "8,9",
        "8,10",
        "9,10",
    ]
    soft_hands = [
        "A,2",
        "A,3",
        "A,4",
        "A,5",
        "A,6",
        "A,7",
        "A,8",
        "A,9",
        "A,10",
    ]
    pairs = [
        "2,2",
        "3,3",
        "4,4",
        "5,5",
        "6,6",
        "7,7",
        "8,8",
        "9,9",
        "10,10",
        "A,A",
    ]
    if subset == "hard":
        cards = hard_hands
    elif subset == "soft":
        cards = soft_hands
    elif subset == "pairs":
        cards = pairs
    elif subset == "hard/soft":
        cards = hard_hands + soft_hands
    elif subset == "soft/pairs":
        cards = soft_hands + pairs
    else:
        raise ValueError("Bad subset")
    card_list = random.choice(cards).split(",")
    return card_list if random.choice([True, False]) else card_list[::-1]



from dataclasses import dataclass
import tkinter
from PIL import Image, ImageTk
import os
from .lib import Card
from argparse import Namespace


IMG_PATH = f"{os.path.dirname(__file__)}/images/"

FOREGROUND = "white"


class TableComponents:
    def __init__(self, root: tkinter.Tk, background: str) -> None:
        self.root = root
        self.background = background
        self._x_slot = 250
        self._padding_left = 20

    def setup_canvas(self) -> None:
        rect = tkinter.Canvas(
            self.root,
            bg=self.background,
            height=100,
            width=80,
            bd=0,
            highlightthickness=0,
            relief="ridge",
        )
        rect.place(x=525, y=485)
        _round_polygon(
            rect,
            [5, 75, 75, 5],
            [5, 5, 90, 90],
            10,
            width=4,
            outline="#bbb500",
            fill=self.background,
        )

    def set_side_panel(self) -> None:
        panel = tkinter.Label(
            self.root,
            width=200,
            height=720,
            background="lightgrey",
            borderwidth=2,
            relief="groove",
        )
        panel.place(x=1000, y=0)

    def get_shoe_status(self) -> tkinter.Label:
        shoe_status_container = tkinter.Label(
            self.root, borderwidth=0, background="white"
        )
        shoe_status_container.place(x=20, y=30, height=150, width=30)
        shoe_progress = tkinter.Label(
            shoe_status_container, background="black", borderwidth=0, anchor="e"
        )
        shoe_label = tkinter.Label(
            self.root,
            text="Discard",
            font="12",
            borderwidth=0,
            background=self.background,
            fg=FOREGROUND,
        )
        shoe_label.place(x=5, y=190)
        return shoe_progress

    def get_label(self) -> tkinter.StringVar:
        label_text = tkinter.StringVar(self.root)
        label = tkinter.Label(
            self.root,
            textvariable=label_text,
            font="Helvetica 13 bold",
            borderwidth=0,
            background=self.background,
            fg=FOREGROUND,
        )
        label.place(x=430, y=670)
        return label_text

    def get_dealer_info(self) -> tkinter.Label:
        dealer_info = tkinter.Label(
            self.root,
            text="",
            font="helvetica 11 bold",
            borderwidth=0,
            background=self.background,
            fg=FOREGROUND,
        )
        dealer_info.place(x=305, y=180)
        return dealer_info

    def get_finger(self) -> dict[str, tkinter.Label]:
        finger = {
            str(slot): tkinter.Label(
                self.root, borderwidth=0, background=self.background
            )
            for slot in range(4)
        }
        for ind, f in enumerate(finger.values()):
            f.place(x=ind * self._x_slot + self._padding_left - 5, y=250)
        return finger

    def get_info(
        self,
    ) -> tuple[dict[str, tkinter.Label], dict[str, tkinter.StringVar]]:
        info_text = {
            str(slot): tkinter.StringVar(self.root) for slot in range(4)
        }
        info = {
            str(slot): tkinter.Label(
                self.root,
                textvariable=info_text[str(slot)],
                font="helvetica 11 bold",
                borderwidth=0,
                background=self.background,
                fg=FOREGROUND,
            )
            for slot in range(4)
        }
        for ind, i in enumerate(info.values()):
            i.place(x=ind * self._x_slot + self._padding_left + 110, y=465)
        return info, info_text

    def get_player_slots(self, n_cards_max: int) -> dict[str, tkinter.Label]:
        slot_player = {
            f"{str(slot)}{str(pos)}": tkinter.Label(
                self.root, borderwidth=0, background=self.background
            )
            for slot in range(4)
            for pos in range(n_cards_max)
        }
        for frame in range(4):
            for pos in range(n_cards_max):
                slot_player[f"{str(frame)}{str(pos)}"].place(
                    x=frame * self._x_slot + pos * 30 + self._padding_left,
                    y=350 - pos * 30,
                )
        return slot_player

    def get_dealer_slot(self) -> dict[str, tkinter.Label]:
        n_cards_max = 11
        card_back_img, _, _ = get_image()
        slot_dealer = {
            f"{str(pos)}": tkinter.Label(
                self.root, borderwidth=0, background=self.background
            )
            for pos in range(n_cards_max)
        }
        for pos in range(2):
            slot_dealer[str(pos)].configure(image=card_back_img)
            slot_dealer[str(pos)].image = card_back_img  # type: ignore
            slot_dealer[str(pos)].pack(side=tkinter.LEFT)
        for pos, slot in enumerate(slot_dealer.values()):
            slot.place(y=40, x=300 + pos * 105)
        return slot_dealer

    def get_chips(self) -> dict[str, tkinter.Label]:
        chips = {
            f"{str(slot)}{str(pos)}": tkinter.Label(
                self.root, borderwidth=0, background=self.background
            )
            for slot in range(4)
            for pos in range(5)
        }

        for a_slot in range(4):
            for pos in range(5):
                padx, pady = 0, 0
                if pos == 1:
                    padx = 50
                elif pos == 2:
                    padx = -50
                elif pos == 3:
                    padx = 100
                elif pos == 4:
                    padx = 25
                    pady = 35
                chips[f"{str(a_slot)}{str(pos)}"].place(
                    x=a_slot * self._x_slot + self._padding_left + padx + 20,
                    y=500 + pady,
                )
        return chips

    def get_insurance_chip(self) -> tkinter.Label:
        insurance_chip = tkinter.Label(
            self.root, borderwidth=0, background=self.background
        )
        insurance_chip.place(x=450, y=400)
        return insurance_chip

    def get_slider(self, x_sidepanel: int, bet: int) -> tkinter.Scale:
        bet_label = tkinter.Label(text="Bet:", background="lightgray")
        slider = tkinter.Scale(
            self.root,
            from_=1,
            to=100,
            orient=tkinter.HORIZONTAL,
            background="lightgray",
        )
        slider.set(bet)
        slider.place(x=x_sidepanel + 40, y=100)
        bet_label.place(x=x_sidepanel, y=120)
        return slider


@dataclass
class CheckConfig:
    location: tuple[int, int]
    txt_location: tuple[int, int]
    txt: str


class CheckButton:
    x = 1020
    y = 580
    y_offset = 25

    def __init__(
        self, root: tkinter.Tk, args: Namespace, background: str
    ) -> None:
        self.root = root
        self.args = args
        self.background = background

    def fetch_count(self) -> tuple[tkinter.StringVar, tkinter.IntVar]:
        txt_location = (10, 610)
        conf = CheckConfig(
            location=(CheckButton.x, CheckButton.y + CheckButton.y_offset),
            txt_location=txt_location,
            txt="Show count",
        )
        label, text = self._get_table_infotext(txt_location)
        checkbox = self._get_checkbutton(label, conf)
        self._show_count(label, checkbox, txt_location)
        return text, checkbox

    def fetch_accuracy(self) -> tuple[tkinter.StringVar, tkinter.IntVar]:
        txt_location = (10, 655)
        conf = CheckConfig(
            location=(CheckButton.x, CheckButton.y),
            txt_location=txt_location,
            txt="Coach mode",
        )
        label, text = self._get_table_infotext(txt_location)
        checkbox = self._get_checkbutton(label, conf)
        self._show_accuracy(label, checkbox, txt_location)
        return text, checkbox

    def fetch_csm(self) -> tkinter.IntVar:
        var = tkinter.IntVar()
        if self.args.rules.region == "Helsinki":
            var.set(1)
        checkbutton = tkinter.Checkbutton(
            self.root,
            text="Continuous shuffle",
            variable=var,
            background="lightgrey",
        )
        checkbutton.place(
            x=CheckButton.x, y=CheckButton.y + 2 * CheckButton.y_offset
        )
        return var

    def _get_table_infotext(
        self, txt_location: tuple[int, int]
    ) -> tuple[tkinter.Label, tkinter.StringVar]:
        text_var = tkinter.StringVar(self.root)
        label = tkinter.Label(
            self.root,
            textvariable=text_var,
            font="Helvetica 10",
            borderwidth=0,
            background=self.background,
            fg=FOREGROUND,
            justify="left",
        )
        label.place(x=txt_location[0], y=txt_location[1])
        return label, text_var

    def _get_checkbutton(
        self, label: tkinter.Label, conf: CheckConfig
    ) -> tkinter.IntVar:
        def toggle():
            if var.get() == 1:
                label.place(x=conf.txt_location[0], y=conf.txt_location[1])
            else:
                label.place_forget()

        var = tkinter.IntVar()
        checkbutton = tkinter.Checkbutton(
            self.root,
            text=conf.txt,
            variable=var,
            background="lightgrey",
            command=toggle,
        )
        checkbutton.place(x=conf.location[0], y=conf.location[1])
        return var

    def _show_accuracy(
        self,
        label: tkinter.Label,
        var: tkinter.IntVar,
        txt_location: tuple[int, int],
    ) -> None:
        if self.args.subset is not None or self.args.cards is not None:
            var.set(1)
        if var.get() == 1:
            label.place(x=txt_location[0], y=txt_location[1])
        else:
            label.place_forget()

    def _show_count(
        self,
        label: tkinter.Label,
        var: tkinter.IntVar,
        txt_location: tuple[int, int],
    ) -> None:
        if self.args.subset is not None or self.args.cards is not None:
            var.set(1)
        if var.get() == 1:
            label.place(x=txt_location[0], y=txt_location[1])
        else:
            label.place_forget()


def get_image(
    card: Card | None = None,
    width: int = 100,
    height: int = 130,
    rotate: bool = False,
):
    if card is None:
        filename = f"{IMG_PATH}/back.png"
    else:
        mapping = {
            "A": "ace",
            "J": "jack",
            "Q": "queen",
            "K": "king",
        }
        prefix = mapping.get(card.label, card.value)
        filename = f"{IMG_PATH}/{prefix}_of_{card.suit}.png"
    image = Image.open(filename).resize(
        (width, height), Image.Resampling.LANCZOS
    )
    if rotate is True:
        image = image.resize((height, height))
        image = image.rotate(angle=90)
        image = image.resize((height, width))
        width, height = height, width
    return ImageTk.PhotoImage(image), width, height


def _round_polygon(canvas, x, y, sharpness, **kwargs):
    sharpness = max(sharpness, 2)
    ratio_multiplier = sharpness - 1
    ratio_divider = sharpness
    points = []
    for i, _ in enumerate(x):
        points.append(x[i])
        points.append(y[i])
        if i != (len(x) - 1):
            points.append((ratio_multiplier * x[i] + x[i + 1]) / ratio_divider)
            points.append((ratio_multiplier * y[i] + y[i + 1]) / ratio_divider)
            points.append((ratio_multiplier * x[i + 1] + x[i]) / ratio_divider)
            points.append((ratio_multiplier * y[i + 1] + y[i]) / ratio_divider)
        else:
            points.append((ratio_multiplier * x[i] + x[0]) / ratio_divider)
            points.append((ratio_multiplier * y[i] + y[0]) / ratio_divider)
            points.append((ratio_multiplier * x[0] + x[i]) / ratio_divider)
            points.append((ratio_multiplier * y[0] + y[i]) / ratio_divider)
            points.append(x[0])
            points.append(y[0])
    return canvas.create_polygon(points, **kwargs, smooth=tkinter.TRUE)
